/*
 * ServerObjectHandler.cpp, part of VCMI engine
 *
 * Authors: listed in file AUTHORS in main folder
 *
 * License: GNU General Public License v2.0 or later
 * Full text of license available in license.txt file, in main folder
 *
 */
 
#include "StdInc.h"
#include "ServerObjectHandler.h"

#include "../lib/CCreatureSet.h"
#include "../lib/CObjectHandler.h"
#include "../lib/NetPacks.h"
#include "../client/CSoundBase.h" //todo: move to common include folder

#include "../lib/CHeroHandler.h"
#include "../lib/CGeneralTextHandler.h"
#include "../lib/CSpellHandler.h"


extern boost::rand48 ran;

///general helper functions

static void sendAndApplyOpenWindow(const OpenWindow::EWindow type, const ui32 id1, const ui32 id2)
{
	OpenWindow ow;
	ow.window = type;
	ow.id1 = id1;
	ow.id2 = id2;
	IObjectInterface::cb->sendAndApply(&ow);
}

namespace server_objects{
	
struct CB
{
	IGameCallback * cb;
	IEventDispatcher * dispatcher;
};


	
///IServerObjectHandler
class IServerObject
{
public:
	IServerObject(CB &cb){this->cb = cb.cb; dispatcher = cb.dispatcher;}
	virtual ~IServerObject(){}
	
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor){}
	virtual void onHeroLeave(const CGObjectInstance *o, const CGHeroInstance *visitor){}
protected:
	IGameCallback *cb;
	IEventDispatcher *dispatcher;
	
	void giveDummyBonus(const CGObjectInstance *o, int heroID, ui8 duration = Bonus::ONE_DAY) const;
};

///HillFort
class HillFort : public IServerObject
{
public:
	HillFort(CB &cb):IServerObject(cb){};
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
};

///Sanctuary
class Sanctuary : public IServerObject
{
public:
	Sanctuary(CB &cb):IServerObject(cb){};
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
};

///Tavern
class Tavern : public IServerObject
{
public:
	Tavern(CB &cb):IServerObject(cb){};
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
};

///Hero instance
class HeroI : public IServerObject
{
public:
	HeroI(CB &cb):IServerObject(cb){};
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
};


///Prison
class Prison : public IServerObject
{
public:
	Prison(CB &cb):IServerObject(cb){};
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
};

///Dwelling
class Dwelling : public IServerObject
{
public:
	Dwelling(CB &cb):IServerObject(cb){};
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
}; 

///Town
class Town : public IServerObject
{
public:
	Town(CB &cb):IServerObject(cb){};
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;
	
private:
	void onHeroVisitInternal(const CGTownInstance *town_obj, const CGHeroInstance *visitor);
	
	void fightOver(const CGTownInstance *o, const CGHeroInstance *h, BattleResult *result);
	void removeCapitols (const CGTownInstance *o,ui8 owner);
};

///VisitableOPH
class VisitableOPH : public IServerObject
{
public:
	VisitableOPH(CB &cb):IServerObject(cb){};
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
private:
	void onNAHeroVisit(const CGVisitableOPH *o, int heroID, bool alreadyVisited);
	
	//handle player's anwer to the Tree of Knowledge dialog
	void treeSelected(const CGVisitableOPH *o, int heroID, int resType, int resVal, TExpType expVal, ui32 result) const; 
	void schoolSelected(const CGVisitableOPH *o, int heroID, ui32 which) const;
	void arenaSelected(const CGVisitableOPH *o, int heroID, int primSkill) const;
};

///Creature
class Creature : public IServerObject
{
public:
	Creature(CB &cb):IServerObject(cb){};	
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;
};

///Mine
class Mine : public IServerObject
{
public:
	Mine(CB &cb):IServerObject(cb){};	
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;
private:
	void endBattle(const CGMine *o, BattleResult *result, ui8 attackingPlayer) const;
	void fight(const CGMine *o, ui32 agreed, const CGHeroInstance *h) const;
	void flagMine(const CGMine *, ui8 player) const;
};

///Resource
class Resource : public IServerObject
{
public:
	Resource(CB &cb):IServerObject(cb){};	
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;
private:
	void collectRes(const CGResource *o, int player) const;
	void fightForRes(const CGResource *o, ui32 agreed, const CGHeroInstance *h) const;
	void endBattle(const CGResource *o, BattleResult *result, const CGHeroInstance *h) const;

};

///VisitableOPW
class VisitableOPW : public IServerObject
{
public:
	VisitableOPW(CB &cb):IServerObject(cb){};	
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
};

///Teleport
class Teleport : public IServerObject
{
public:
	Teleport(CB &cb):IServerObject(cb){};	
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
};

///Artifact
class Artifact : public IServerObject
{
public:
	Artifact(CB &cb):IServerObject(cb){};	
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
private:
	void fightForArt(const CGArtifact *o, ui32 agreed, const CGHeroInstance *h) const;
	void endBattle(const CGArtifact *o, BattleResult *result, const CGHeroInstance *h) const;
	void pick(const CGArtifact *o, const CGHeroInstance *h) const;
  
};

///Pickable
class Pickable : public IServerObject
{
public:
	Pickable(CB &cb):IServerObject(cb){}
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
private:
	void chosen(const CGPickable *o, int which, int heroID) const;
};

///SeerHut
class SeerHut : public IServerObject
{
	//TODO:SeerHut
};

///WitchHut
class WitchHut : public IServerObject
{
	//TODO:WitchHut
};

///BonusingObject
class BonusingObject : public IServerObject
{
public:
	BonusingObject(CB &cb):IServerObject(cb){}
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
};

///MagicSpring
class MagicSpring : public IServerObject
{
public:
	MagicSpring(CB &cb):IServerObject(cb){}
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
};

///MagicWell
class MagicWell : public IServerObject
{
public:
	MagicWell(CB &cb):IServerObject(cb){}
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
};

///CGPandoraBox
class PandoraBox : public IServerObject
{
public:
	PandoraBox(CB &cb):IServerObject(cb){}
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
protected:	
	void giveContents(const CGPandoraBox *o, const CGHeroInstance *h, bool afterBattle) const;
	void endBattle(const CGPandoraBox *o, const CGHeroInstance *h, BattleResult *result) const;
private:
	void open (const CGPandoraBox *o, const CGHeroInstance *h, ui32 accept) const;
	

};

///CGEvent
class Event : public PandoraBox
{
public:
	Event(CB &cb):PandoraBox(cb){}
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;
private:
	void activated(const CGEvent *o, const CGHeroInstance * h) const;
};

///CGObservatory
class Observatory : public IServerObject
{
public:
	Observatory(CB &cb):IServerObject(cb){}
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
};

///CGShrine
class Shrine : public IServerObject
{
public:
	Shrine(CB &cb):IServerObject(cb){}
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
};

///CGSignBottle
class SignBottle : public IServerObject
{
public:
	SignBottle(CB &cb):IServerObject(cb){}
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
};

///CGScholar
class Scholar : public IServerObject
{
public:
	Scholar(CB &cb):IServerObject(cb){}
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
};

///CGGarrison
class Garrison : public IServerObject
{
public:
	Garrison(CB &cb):IServerObject(cb){}
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
private:
	void fightOver (const CGGarrison *o, const CGHeroInstance *h, BattleResult *result);
};

///CGOnceVisitable
class OnceVisitable : public IServerObject
{
public:
	OnceVisitable(CB &cb):IServerObject(cb){}
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;	
	
private:
	void searchTomb(const CGOnceVisitable *o, const CGHeroInstance *h, ui32 accept) const;
};

///CBank
class Bank : public IServerObject
{
public:
	Bank(CB &cb):IServerObject(cb){}
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;
	
protected:
	void fightGuards (const CBank *o, const CGHeroInstance *h, ui32 accept);
	virtual void endBattle (const CBank *o, const CGHeroInstance *h, const BattleResult *result);
};

///CGPyramid
class Pyramid : public Bank
{
public:
	Pyramid(CB &cb):Bank(cb){}
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;
protected:
	virtual void endBattle (const CBank *o, const CGHeroInstance *h, const BattleResult *result) override;
};



///CEventDispatcher
class CEventDispatcher : public IEventDispatcher
{
public:
	CEventDispatcher(IGameCallback * cb);
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) override;
	
private: 
	/**
	* map ObjectID -> handler
	*/
	
	typedef shared_ptr<IServerObject> THandlerPtr; 
	
	std::map<Obj::Obj,THandlerPtr> handlers;
};

void IServerObject::giveDummyBonus(const CGObjectInstance *o,int heroID, ui8 duration) const
{
	GiveBonus gbonus;
	gbonus.bonus.type = Bonus::NONE;
	gbonus.id = heroID;
	gbonus.bonus.duration = duration;
	gbonus.bonus.source = Bonus::OBJECT;
	gbonus.bonus.sid = o->ID;
	cb->giveHeroBonus(&gbonus);
}



///HillFort
void HillFort::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	sendAndApplyOpenWindow(OpenWindow::HILL_FORT_WINDOW,o->id,visitor->id);
}

///Sanctuary
void Sanctuary::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	InfoWindow iw;
	iw.player = visitor->tempOwner;
	iw.soundID = soundBase::GETPROTECTION;
	iw.text.addTxt(MetaString::ADVOB_TXT, 114);  //You enter the sanctuary and immediately feel as if a great weight has been lifted off your shoulders.  You feel safe here.
	cb->sendAndApply(&iw);
}


///Tavern
void Tavern::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	sendAndApplyOpenWindow(OpenWindow::TAVERN_WINDOW,visitor->id,o->id);
}


///Hero instance
void HeroI::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	auto hero_obj = dynamic_cast<const CGHeroInstance *>(o);
	if( cb->gameState()->getPlayerRelations(hero_obj->tempOwner, visitor->tempOwner)) //our or ally hero
	{
		//exchange
		cb->heroExchange(visitor->id, hero_obj->id);
	}
	else //battle
	{
		if(hero_obj->visitedTown) //we're in town
			dispatcher->onHeroVisit(hero_obj->visitedTown,visitor);//town will handle attacking 
		else
			cb->startBattleI(visitor,hero_obj);
	}		
}

///Prison
void Prison::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	InfoWindow iw;
	iw.player = visitor->tempOwner;
	iw.soundID = soundBase::ROGUE;

	if(cb->getHeroCount(visitor->tempOwner,false) < 8) //free hero slot
	{
		cb->changeObjPos(o->id,o->pos+int3(1,0,0),0);
		cb->setObjProperty(o->id, ObjProperty::ID, Obj::HERO); //set object type to HeroI
		cb->giveHero(o->id,visitor->tempOwner); //recreates def and adds hero to player

		iw.text << std::pair<ui8,ui32>(11,102);
	}
	else //already 8 wandering heroes
	{
		iw.text << std::pair<ui8,ui32>(11,103);
	}

	cb->showInfoDialog(&iw);	
}


///Town
void Town::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	onHeroVisitInternal(dynamic_cast<const CGTownInstance*>(o),visitor);
}

void Town::onHeroVisitInternal(const CGTownInstance *town_obj, const CGHeroInstance* visitor)
{
	if( !cb->gameState()->getPlayerRelations( town_obj->getOwner(), visitor->getOwner() ))//if this is enemy
	{
		if(town_obj->armedGarrison() || town_obj->visitingHero)
		{
			const CGHeroInstance *defendingHero = nullptr;
			const CArmedInstance *defendingArmy = town_obj;

			if(town_obj->visitingHero)
				defendingHero = town_obj->visitingHero;
			else if(town_obj->garrisonHero)
				defendingHero = town_obj->garrisonHero;

			if(defendingHero)
				defendingArmy = defendingHero;

			bool outsideTown = (defendingHero == town_obj->visitingHero && town_obj->garrisonHero);

			//TODO
			//"borrowing" army from garrison to visiting hero

			cb->startBattleI(visitor, defendingArmy, town_obj->getSightCenter(), visitor, defendingHero, false, boost::bind(&Town::fightOver,this,town_obj, visitor, _1), (outsideTown ? nullptr : town_obj));
		}
		else
		{
			cb->setOwner(town_obj->id, visitor->tempOwner);
			town_obj->removeCapitols(visitor->getOwner());
			cb->heroVisitCastle(town_obj->id, visitor->id);
		}
	}
	else
	{
		if (visitor->commander && !visitor->commander->alive) //rise commander. TODO: interactive script
		{
			SetCommanderProperty scp;
			scp.heroid = visitor->id;
			scp.which = SetCommanderProperty::ALIVE;
			scp.amount = 1;
			cb->sendAndApply (&scp);
		}
		cb->heroVisitCastle(town_obj->id, visitor->id);
	}
}

void Town::fightOver(const CGTownInstance* o, const CGHeroInstance* h, BattleResult* result)
{
	if(result->winner == 0)
	{
		removeCapitols (o, h->getOwner());
		cb->setOwner (o->id, h->tempOwner); //give control after checkout is done
		FoWChange fw;
		fw.player = h->tempOwner;
		fw.mode = 1;
		o->getSightTiles (fw.tiles); //update visibility for castle structures
		cb->sendAndApply (&fw);
	}	
}

void Town::removeCapitols(const CGTownInstance* o, ui8 owner)
{
	if (o->hasCapitol()) // search if there's an older capitol
	{
		PlayerState* state = cb->gameState()->getPlayer (owner); //get all towns owned by player
		
		BOOST_FOREACH (auto i, state->towns)
		{
			if (i != o && (i)->hasCapitol())
			{
				RazeStructures rs;
				rs.tid = o->id;
				rs.bid.insert(13);
				rs.destroyed = o->destroyed;
				cb->sendAndApply(&rs);
				return;
			}
		}
	}
}


void Dwelling::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	auto dwel_obj = dynamic_cast<const CGDwelling*>(o);
	if(dwel_obj->ID == Obj::REFUGEE_CAMP && !dwel_obj->creatures[0].first) //Refugee Camp, no available cres
	{
		InfoWindow iw;
		iw.player = visitor->tempOwner;
		iw.text.addTxt(MetaString::ADVOB_TXT, 44); //{%s} \n\n The camp is deserted.  Perhaps you should try next week.
		iw.text.addReplacement(MetaString::OBJ_NAMES, dwel_obj->ID);
		cb->sendAndApply(&iw);
		return;
	}

	int relations = cb->gameState()->getPlayerRelations( visitor->tempOwner, dwel_obj->tempOwner );

	if ( relations == 1 )//ally
		return;//do not allow recruiting or capturing

	if( !relations  &&  dwel_obj->stacksCount() > 0) //object is guarded, owned by enemy
	{
		BlockingDialog bd;
		bd.player = visitor->tempOwner;
		bd.flags = BlockingDialog::ALLOW_CANCEL;
		bd.text.addTxt(MetaString::GENERAL_TXT, 421); //Much to your dismay, the %s is guarded by %s %s. Do you wish to fight the guards?
		bd.text.addReplacement(dwel_obj->ID == Obj::CREATURE_GENERATOR1 ? MetaString::CREGENS : MetaString::CREGENS4, dwel_obj->subID);
		bd.text.addReplacement(MetaString::ARRAY_TXT, 176 + dwel_obj->Slots().begin()->second->getQuantityID()*3);
		bd.text.addReplacement(*(dwel_obj->Slots()).begin()->second);
		cb->showBlockingDialog(&bd, boost::bind(&CGDwelling::wantsFight, dwel_obj, visitor, _1));//TODO: use local function
		return;
	}

	if(!relations  &&  dwel_obj->ID != Obj::WAR_MACHINE_FACTORY)
	{
		cb->setOwner(dwel_obj->id, visitor->tempOwner);
	}

	BlockingDialog bd;
	bd.player = visitor->tempOwner;
	bd.flags = BlockingDialog::ALLOW_CANCEL;

	switch(dwel_obj->ID)
	{
	case Obj::CREATURE_GENERATOR1:
	case Obj::CREATURE_GENERATOR4:
		{
			bd.text.addTxt(MetaString::ADVOB_TXT, dwel_obj->ID == Obj::CREATURE_GENERATOR1 ? 35 : 36); //{%s} Would you like to recruit %s? / {%s} Would you like to recruit %s, %s, %s, or %s?
			bd.text.addReplacement(dwel_obj->ID == Obj::CREATURE_GENERATOR1 ? MetaString::CREGENS : MetaString::CREGENS4, dwel_obj->subID);
			for(size_t i = 0; i < dwel_obj->creatures.size(); i++)
				bd.text.addReplacement(MetaString::CRE_PL_NAMES, dwel_obj->creatures[i].second[0]);				
		}
		break;
	case Obj::REFUGEE_CAMP:
		{
			bd.text.addTxt(MetaString::ADVOB_TXT, 35); //{%s} Would you like to recruit %s?
			bd.text.addReplacement(MetaString::OBJ_NAMES, dwel_obj->ID);
			for(size_t i = 0; i < dwel_obj->creatures.size(); i++)
				bd.text.addReplacement(MetaString::CRE_PL_NAMES, dwel_obj->creatures[i].second[0]);	
		}
		break;
	case Obj::WAR_MACHINE_FACTORY:
		{
			bd.text.addTxt(MetaString::ADVOB_TXT, 157); //{War Machine Factory} Would you like to purchase War Machines?
		}
		break;
	default:
		{
			throw std::runtime_error("Illegal dwelling!");
		}
	}

	cb->showBlockingDialog(&bd, boost::bind(&CGDwelling::heroAcceptsCreatures, dwel_obj, visitor, _1));//TODO: use local function

}


///VisitableOPH


void VisitableOPH::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	const CGVisitableOPH * oph_obj = dynamic_cast<const CGVisitableOPH*>(o);
	if(oph_obj->visitors.find(visitor->id)==oph_obj->visitors.end())
	{
		onNAHeroVisit(oph_obj, visitor->id, false);
		switch(o->ID)
		{
		case Obj::TREE_OF_KNOWLEDGE:
		case Obj::ARENA:
		case Obj::LIBRARY_OF_ENLIGHTENMENT:
		case Obj::SCHOOL_OF_MAGIC:
		case Obj::SCHOOL_OF_WAR:
			break;
		default:
			cb->setObjProperty(o->id, ObjProperty::VISITORS, visitor->id); //add to the visitors
			break;
		}
	}
	else
	{
		onNAHeroVisit(oph_obj, visitor->id, true);
	}	
}

void VisitableOPH::onNAHeroVisit(const CGVisitableOPH * o, int heroID, bool alreadyVisited)
{
	Component::EComponentType id = (Component::EComponentType)0;
	int subid=0, ot=0, sound = 0;
	TExpType val=1;
	switch(o->ID)
	{
	case Obj::ARENA:
		sound = soundBase::NOMAD;
		ot = 0;
		break;
	case Obj::MERCENARY_CAMP:
		sound = soundBase::NOMAD;
		subid=0;
		ot=80;
		break;
	case Obj::MARLETTO_TOWER:
		sound = soundBase::NOMAD;
		subid=1;
		ot=39;
		break;
	case Obj::STAR_AXIS:
		sound = soundBase::gazebo;
		subid=2;
		ot=100;
		break;
	case Obj::GARDEN_OF_REVELATION:
		sound = soundBase::GETPROTECTION;
		subid=3;
		ot=59;
		break;
	case Obj::LEARNING_STONE:
		sound = soundBase::gazebo;
		id=Component::EXPERIENCE;
		ot=143;
		val=1000;
		break;
	case Obj::TREE_OF_KNOWLEDGE:
		sound = soundBase::gazebo;
		id = Component::EXPERIENCE;
		subid = 1;
		ot = 146;
		val = 1;
		break;
	case Obj::LIBRARY_OF_ENLIGHTENMENT:
		sound = soundBase::gazebo;
		ot = 66;
		break;
	case Obj::SCHOOL_OF_MAGIC:
		sound = soundBase::faerie;
		ot = 71;
		break;
	case Obj::SCHOOL_OF_WAR:
		sound = soundBase::MILITARY;
		ot = 158;
		break;
	}
	if (!alreadyVisited)
	{
		switch (o->ID)
		{
		case Obj::ARENA:
			{
				BlockingDialog sd(false,true);
				sd.soundID = sound;
				sd.text << std::pair<ui8,ui32>(11,ot);
				sd.components.push_back(Component(Component::PRIM_SKILL, PrimarySkill::ATTACK, 2, 0));
				sd.components.push_back(Component(Component::PRIM_SKILL, PrimarySkill::DEFENSE, 2, 0));
				sd.player = cb->getOwner(heroID);
				cb->showBlockingDialog(&sd,boost::bind(&VisitableOPH::arenaSelected,this,o,heroID,_1));
				return;
			}
		case Obj::MERCENARY_CAMP:
		case Obj::MARLETTO_TOWER:
		case Obj::STAR_AXIS:
		case Obj::GARDEN_OF_REVELATION:
			{
				cb->changePrimSkill(heroID,subid,val);
				InfoWindow iw;
				iw.soundID = sound;
				iw.components.push_back(Component(Component::PRIM_SKILL, subid,val,0));
				iw.text << std::pair<ui8,ui32>(11,ot);
				iw.player = cb->getOwner(heroID);
				cb->showInfoDialog(&iw);
				break;
			}
		case Obj::LEARNING_STONE: //give exp
			{
				const CGHeroInstance *h = cb->getHero(heroID);
				val = h->calculateXp(val);
				InfoWindow iw;
				iw.soundID = sound;
				iw.components.push_back(Component(id,subid,val,0));
				iw.player = cb->getOwner(heroID);
				iw.text << std::pair<ui8,ui32>(11,ot);
				iw.soundID = soundBase::gazebo;
				cb->showInfoDialog(&iw);
				cb->changePrimSkill(heroID,4,val);
				break;
			}
		case Obj::TREE_OF_KNOWLEDGE:
			{
				const CGHeroInstance *h = cb->getHero(heroID);
				val = VLC->heroh->reqExp(h->level+val) - VLC->heroh->reqExp(h->level);
				if(!o->ttype)
				{
					cb->setObjProperty(o->id, ObjProperty::VISITORS, heroID); //add to the visitors
					InfoWindow iw;
					iw.soundID = sound;
					iw.components.push_back(Component(id,subid,1,0));
					iw.player = cb->getOwner(heroID);
					iw.text << std::pair<ui8,ui32>(11,148);
					cb->showInfoDialog(&iw);
					cb->changePrimSkill(heroID,4,val);
					break;
				}
				else
				{
					ui32 res;
					TExpType resval;
					if(o->ttype==1)
					{
						res = 6;
						resval = 2000;
						ot = 149;
					}
					else
					{
						res = 5;
						resval = 10;
						ot = 151;
					}

					if(cb->getResource(h->tempOwner,res) < resval) //not enough resources
					{
						ot++;
						InfoWindow iw;
						iw.soundID = sound;
						iw.player = h->tempOwner;
						iw.text << std::pair<ui8,ui32>(11,ot);
						cb->showInfoDialog(&iw);
						return;
					}

					BlockingDialog sd (true, false);
					sd.soundID = sound;
					sd.player = cb->getOwner(heroID);
					sd.text << std::pair<ui8,ui32>(11,ot);
					sd.components.push_back (Component (Component::RESOURCE, res, resval, 0));
					cb->showBlockingDialog(&sd,boost::bind(&VisitableOPH::treeSelected,this,o,heroID,res,resval,val,_1));
				}
				break;
			}
		case Obj::LIBRARY_OF_ENLIGHTENMENT:
			{
				const CGHeroInstance *h = cb->getHero(heroID);
				if(h->level  <  10 - 2*h->getSecSkillLevel(CGHeroInstance::DIPLOMACY)) //not enough level
				{
					InfoWindow iw;
					iw.soundID = sound;
					iw.player = cb->getOwner(heroID);
					iw.text << std::pair<ui8,ui32>(11,68);
					cb->showInfoDialog(&iw);
				}
				else
				{
					cb->setObjProperty(o->id, ObjProperty::VISITORS, heroID); //add to the visitors
					cb->changePrimSkill(heroID,0,2);
					cb->changePrimSkill(heroID,1,2);
					cb->changePrimSkill(heroID,2,2);
					cb->changePrimSkill(heroID,3,2);
					InfoWindow iw;
					iw.soundID = sound;
					iw.player = cb->getOwner(heroID);
					iw.text << std::pair<ui8,ui32>(11,66);
					cb->showInfoDialog(&iw);
				}
				break;
			}
		case Obj::SCHOOL_OF_MAGIC:
		case Obj::SCHOOL_OF_WAR:
			{
				int skill = (o->ID==Obj::SCHOOL_OF_MAGIC ? 2 : 0);
				if(cb->getResource(cb->getOwner(heroID),6) < 1000) //not enough resources
				{
					InfoWindow iw;
					iw.soundID = sound;
					iw.player = cb->getOwner(heroID);
					iw.text << std::pair<ui8,ui32>(MetaString::ADVOB_TXT,ot+2);
					cb->showInfoDialog(&iw);
				}
				else
				{
					BlockingDialog sd(true,true);
					sd.soundID = sound;
					sd.player = cb->getOwner(heroID);
					sd.text << std::pair<ui8,ui32>(11,ot);
					sd.components.push_back(Component(Component::PRIM_SKILL, skill, +1, 0));
					sd.components.push_back(Component(Component::PRIM_SKILL, skill+1, +1, 0));
					cb->showBlockingDialog(&sd,boost::bind(&VisitableOPH::schoolSelected,this,o,heroID,_1));
				}
			}
			break;
		}
	}
	else
	{
		ot++;
		InfoWindow iw;
		iw.soundID = sound;
		iw.player = cb->getOwner(heroID);
		iw.text << std::pair<ui8,ui32>(11,ot);
		cb->showInfoDialog(&iw);
	}
}

void VisitableOPH::treeSelected(const CGVisitableOPH* o, int heroID, int resType, int resVal, TExpType expVal, ui32 result) const
{
	if(result) //player agreed to give res for exp
	{
		cb->giveResource(cb->getOwner(heroID),resType,-resVal); //take resource
		cb->changePrimSkill(heroID,4,expVal); //give exp
		cb->setObjProperty(o->id, ObjProperty::VISITORS, heroID); //add to the visitors
	}	
}

void VisitableOPH::schoolSelected(const CGVisitableOPH* o, int heroID, ui32 which) const
{
	if(!which) //player refused to pay
		return;

	int base = (o->ID == Obj::SCHOOL_OF_MAGIC  ?  2  :  0);
	cb->setObjProperty(o->id, ObjProperty::VISITORS, heroID); //add to the visitors
	cb->giveResource(cb->getOwner(heroID),6,-1000); //take 1000 gold
	cb->changePrimSkill(heroID, base + which-1, +1); //give appropriate skill	
}

void VisitableOPH::arenaSelected(const CGVisitableOPH* o, int heroID, int primSkill) const
{
	cb->setObjProperty(o->id, ObjProperty::VISITORS, heroID); //add to the visitors
	cb->changePrimSkill(heroID,primSkill-1,2);	
}

///Creature
void Creature::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	const CGCreature * co = dynamic_cast<const CGCreature*>(o);
	int action = co->takenAction(visitor);
	switch( action ) //decide what we do...
	{
	case -2: //fight
		co->fight(visitor); //TODO: use local function
		break;
	case -1: //flee
		{
			co->flee(visitor);//TODO: use local function
			break;
		}
	case 0: //join for free
		{
			BlockingDialog ynd(true,false);
			ynd.player = visitor->tempOwner;
			ynd.text << std::pair<ui8,ui32>(MetaString::ADVOB_TXT, 86);
			ynd.text.addReplacement(MetaString::CRE_PL_NAMES, co->subID);
			cb->showBlockingDialog(&ynd,boost::bind(&CGCreature::joinDecision,co,visitor,0,_1));//TODO: use local function
			break;
		}
	default: //join for gold
		{
			assert(action > 0);

			//ask if player agrees to pay gold
			BlockingDialog ynd(true,false);
			ynd.player = visitor->tempOwner;
			std::string tmp = VLC->generaltexth->advobtxt[90];
			boost::algorithm::replace_first(tmp,"%d",boost::lexical_cast<std::string>(co->getStackCount(0)));
			boost::algorithm::replace_first(tmp,"%d",boost::lexical_cast<std::string>(action));
			boost::algorithm::replace_first(tmp,"%s",VLC->creh->creatures[co->subID]->namePl);
			ynd.text << tmp;
			cb->showBlockingDialog(&ynd,boost::bind(&CGCreature::joinDecision,co,visitor,action,_1));//TODO: use local function
			break;
		}
	}
}

//Mine
void Mine::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	const CGMine * obj = dynamic_cast<const CGMine*>(o);
	
	int relations = cb->gameState()->getPlayerRelations(visitor->tempOwner, o->tempOwner);

	if(relations == 2) //we're visiting our mine
	{
		cb->showGarrisonDialog(o->id,visitor->id,true,0);
		return;
	}
	else if (relations == 1)//ally
		return;

	if(obj->stacksCount()) //Mine is guarded
	{
		BlockingDialog ynd(true,false);
		ynd.player = visitor->tempOwner;
		ynd.text << std::pair<ui8,ui32>(MetaString::ADVOB_TXT, o->subID == 7 ? 84 : 187);
		cb->showBlockingDialog(&ynd,boost::bind(&Mine::fight, this, obj, _1, visitor));
		return;
	}

	obj->flagMine(visitor->tempOwner);
}

void Mine::endBattle(const CGMine* o, BattleResult* result, ui8 attackingPlayer) const
{
	if(result->winner == 0) //attacker won
	{
		if(o->subID == 7)
		{
			InfoWindow iw;
			iw.player = attackingPlayer;
			iw.text.addTxt(MetaString::ADVOB_TXT, 85);
			cb->showInfoDialog(&iw);

		}
		flagMine(o, attackingPlayer);
	}	
}

void Mine::fight(const CGMine* o, ui32 agreed, const CGHeroInstance* h) const
{
	cb->startBattleI(h, o, boost::bind(&Mine::endBattle, this, o, _1, h->tempOwner));
}

void Mine::flagMine(const CGMine* o, ui8 player) const
{
	assert(o->tempOwner != player);
	cb->setOwner(o->id,player); //not ours? flag it!

	MetaString ms;
	ms << std::pair<ui8,ui32>(9,o->subID) << "\n(" << std::pair<ui8,ui32>(6,23+player) << ")";
	if(o->subID == 7)
	{
		ms << "(%s)";
		ms.addReplacement(MetaString::RES_NAMES, o->producedResource);
	}
	cb->setHoverName(o->id,&ms);

	InfoWindow iw;
	iw.soundID = soundBase::FLAGMINE;
	iw.text.addTxt(MetaString::MINE_EVNTS,o->producedResource); //not use subID, abandoned mines uses default mine texts
	iw.player = player;
	iw.components.push_back(Component(Component::RESOURCE,o->producedResource,o->producedQuantity,-1));
	cb->showInfoDialog(&iw);	
}

///Resource
void Resource::onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor)
{
	const CGResource * obj = dynamic_cast<const CGResource*>(o);
	if(obj->stacksCount())
	{
		if(obj->message.size())
		{
			BlockingDialog ynd(true,false);
			ynd.player = visitor->getOwner();
			ynd.text << obj->message;
			cb->showBlockingDialog(&ynd,boost::bind(&Resource::fightForRes,this, obj, _1,visitor));
		}
		else
		{
			fightForRes(obj,1,visitor);
		}
	}
	else
	{
		if(obj->message.length())
		{
			InfoWindow iw;
			iw.player = visitor->tempOwner;
			iw.text << obj->message;
			cb->showInfoDialog(&iw);
		}
		collectRes(obj,visitor->getOwner());
	}
	
}

void Resource::collectRes(const CGResource* o, int player) const
{
	cb->giveResource(player,o->subID,o->amount);
	ShowInInfobox sii;
	sii.player = player;
	sii.c = Component(Component::RESOURCE,o->subID,o->amount,0);
	sii.text << std::pair<ui8,ui32>(11,113);
	sii.text.addReplacement(MetaString::RES_NAMES, o->subID);
	cb->showCompInfo(&sii);
	cb->removeObject(o->id);	
}

void Resource::fightForRes(const CGResource* o, ui32 agreed, const CGHeroInstance* h) const
{
	if(agreed)
		cb->startBattleI(h, o, boost::bind(&Resource::endBattle,this, o, _1,h));	
}

void Resource::endBattle(const CGResource* o, BattleResult* result, const CGHeroInstance* h) const
{
	if(result->winner == 0) //attacker won
		collectRes(o,h->getOwner());
}

///VisitableOPW
void VisitableOPW::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	auto obj = dynamic_cast<const CGVisitableOPW *>(o);
	
	int mid=0, sound = 0;
	switch (o->ID)
	{
	case Obj::MYSTICAL_GARDEN:
		sound = soundBase::experience;
		mid = 92;
		break;
	case Obj::WINDMILL:
		sound = soundBase::GENIE;
		mid = 170;
		break;
	case Obj::WATER_WHEEL:
		sound = soundBase::GENIE;
		mid = 164;
		break;
	default:
		assert(0);
	}
	if (obj->visited)
	{
		if (o->ID!=Obj::WINDMILL)
			mid++;
		else
			mid--;

		InfoWindow iw;
		iw.soundID = sound;
		iw.player = visitor->tempOwner;
		iw.text << std::pair<ui8,ui32>(11,mid);
		cb->showInfoDialog(&iw);
	}
	else
	{
		Component::EComponentType type = Component::RESOURCE;
		int sub=0, val=0;

		switch (o->ID)
		{
		case Obj::MYSTICAL_GARDEN:
			if (rand()%2)
			{
				sub = 5;
				val = 5;
			}
			else
			{
				sub = 6;
				val = 500;
			}
			break;
		case Obj::WINDMILL:
			mid = 170;
			sub = (rand() % 5) + 1;
			val = (rand() % 4) + 3;
			break;
		case Obj::WATER_WHEEL:
			mid = 164;
			sub = 6;
			if(cb->getDate(0)<8)
				val = 500;
			else
				val = 1000;
		}
		cb->giveResource(visitor->tempOwner,sub,val);
		InfoWindow iw;
		iw.soundID = sound;
		iw.player = visitor->tempOwner;
		iw.components.push_back(Component(type,sub,val,0));
		iw.text << std::pair<ui8,ui32>(11,mid);
		cb->showInfoDialog(&iw);
		cb->setObjProperty(o->id, ObjProperty::VISITED, true);
		MetaString ms; //set text to "visited"
		ms << std::pair<ui8,ui32>(3,o->ID) << " " << std::pair<ui8,ui32>(1,352);
		cb->setHoverName(o->id,&ms);
	}	
}

void Teleport::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	int destinationid=-1;
	
	const auto id = o->id;
	const auto subID = o->subID;
	const auto ID = o->ID;
	const auto h = visitor;

	
	auto obj = dynamic_cast<const CGTeleport *>(o);
	
	switch(ID)
	{
	case Obj::MONOLITH1: //one way - find corresponding exit monolith
		if(vstd::contains(CGTeleport::objs,Obj::MONOLITH2) && vstd::contains(CGTeleport::objs[Obj::MONOLITH2],subID) && CGTeleport::objs[Obj::MONOLITH2][subID].size())
			destinationid = CGTeleport::objs[Obj::MONOLITH2][subID][rand()%CGTeleport::objs[Obj::MONOLITH2][subID].size()];
		else
			tlog2 << "Cannot find corresponding exit monolith for "<< id << std::endl;
		break;
	case Obj::MONOLITH3://two way monolith - pick any other one
	case Obj::WHIRLPOOL: //Whirlpool
		if(vstd::contains(CGTeleport::objs,ID) && vstd::contains(CGTeleport::objs[ID],subID) && CGTeleport::objs[ID][subID].size()>1)
		{
			while ((destinationid = CGTeleport::objs[ID][subID][rand()%CGTeleport::objs[ID][subID].size()]) == id); //choose another exit
			if (ID == Obj::WHIRLPOOL)
			{
				if (!h->hasBonusOfType(Bonus::WHIRLPOOL_PROTECTION))
				{
					if (h->Slots().size() > 1 || h->Slots().begin()->second->count > 1)
					{ //we can't remove last unit
						TSlot targetstack = h->Slots().begin()->first; //slot numbers may vary
						for(TSlots::const_reverse_iterator i = h->Slots().rbegin(); i != h->Slots().rend(); i++)
						{
							if (h->getPower(targetstack) > h->getPower(i->first))
							{
								targetstack = (i->first);
							}
						}

						TQuantity countToTake = h->getStackCount(targetstack) * 0.5;
						vstd::amax(countToTake, 1);


						InfoWindow iw;
						iw.player = h->tempOwner;
						iw.text.addTxt (MetaString::ADVOB_TXT, 168);
						iw.components.push_back (Component(CStackBasicDescriptor(h->getCreature(targetstack), countToTake)));
						cb->showInfoDialog(&iw);
						cb->changeStackCount(StackLocation(h, targetstack), -countToTake);
					}
				}
			}
		}
		else
			tlog2 << "Cannot find corresponding exit monolith for "<< id << std::endl;
		break;
	case Obj::SUBTERRANEAN_GATE: //find nearest subterranean gate on the other level
		{
			destinationid = obj->getMatchingGate(id);
			if(destinationid < 0) //no exit
			{
				InfoWindow iw;
				iw.player = h->tempOwner;
				iw.text.addTxt(MetaString::ADVOB_TXT, 153);//Just inside the entrance you find a large pile of rubble blocking the tunnel. You leave discouraged.
				cb->sendAndApply(&iw);
			}
			break;
		}
	}
	if(destinationid < 0)
	{
		tlog2 << "Cannot find exit... (obj at " << o->pos << ") :( \n";
		return;
	}
	if (ID == Obj::WHIRLPOOL)
	{
		std::set<int3> tiles = cb->getObj(destinationid)->getBlockedPos();
		std::set<int3>::iterator it = tiles.begin();
		std::advance (it, rand() % tiles.size()); //picking random element of set is tiring
		cb->moveHero (h->id, *it + int3(1,0,0), true);
	}
	else
		cb->moveHero (h->id,CGHeroInstance::convertPosition(cb->getObj(destinationid)->pos,true) - o->getVisitableOffset(), true);
}

///Artifact
void Artifact::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	const auto subID = o->subID;
	const auto ID = o->ID;
	const auto h = visitor;
	
	auto obj = dynamic_cast<const CGArtifact *>(o);
	auto message = obj->message;
	
	if(!obj->stacksCount())
	{
		InfoWindow iw;
		iw.player = h->tempOwner;
		switch(ID)
		{
		case Obj::ARTIFACT:
			{
				iw.soundID = soundBase::treasure; //play sound only for non-scroll arts
				iw.components.push_back(Component(Component::ARTIFACT,subID,0,0));
				if(message.length())
					iw.text <<  message;
				else
					iw.text << std::pair<ui8, ui32> (MetaString::ART_EVNTS, subID);
			}
			break;
		case Obj::SPELL_SCROLL:
			{
				int spellID = obj->storedArtifact->getGivenSpellID();
				iw.components.push_back (Component(Component::SPELL, spellID,0,0));
				iw.text.addTxt (MetaString::ADVOB_TXT,135);
				iw.text.addReplacement(MetaString::SPELL_NAME, spellID);
			}
			break;
		}
		cb->showInfoDialog(&iw);
		pick(obj, h);
	}
	else
	{
		if(message.size())
		{
			BlockingDialog ynd(true,false);
			ynd.player = h->getOwner();
			ynd.text << message;
			cb->showBlockingDialog(&ynd,boost::bind(&Artifact::fightForArt,this,obj,_1,h));
		}
		else
		{
			fightForArt(obj,0,h);
		}
	}
}

void Artifact::fightForArt(const CGArtifact* o, ui32 agreed, const CGHeroInstance* h) const
{
	if(agreed)
		cb->startBattleI(h, o, boost::bind(&Artifact::endBattle,this,o,_1,h));	
}

void Artifact::endBattle(const CGArtifact* o, BattleResult* result, const CGHeroInstance* h) const
{
	if(result->winner == 0) //attacker won
		pick(o,h);	
}

void Artifact::pick(const CGArtifact* o, const CGHeroInstance* h) const
{
	cb->giveHeroArtifact(h, o->storedArtifact, -2);
	cb->removeObject(o->id);	
}


void Pickable::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	const auto subID = o->subID;
	const auto ID = o->ID;
	const auto h = visitor;
	const auto id = o->id;
	
	auto obj = dynamic_cast<const CGPickable *>(o);
	
	auto val1 = obj->val1;
	auto val2 = obj->val2;
	auto type = obj->type;
	
	
	switch(ID)
	{
	case Obj::CAMPFIRE:
		{
			cb->giveResource(h->tempOwner,type,val2); //non-gold resource
			cb->giveResource(h->tempOwner,Res::GOLD,val1);//gold
			InfoWindow iw;
			iw.soundID = soundBase::experience;
			iw.player = h->tempOwner;
			iw.components.push_back(Component(Component::RESOURCE,Res::GOLD,val1,0));
			iw.components.push_back(Component(Component::RESOURCE,type,val2,0));
			iw.text << std::pair<ui8,ui32>(11,23);
			cb->showInfoDialog(&iw);
			break;
		}
	case Obj::FLOTSAM:
		{
			cb->giveResource(h->tempOwner,0,val1); //wood
			cb->giveResource(h->tempOwner,6,val2);//gold
			InfoWindow iw;
			iw.soundID = soundBase::GENIE;
			iw.player = h->tempOwner;
			if(val1)
				iw.components.push_back(Component(Component::RESOURCE,Res::WOOD,val1,0));
			if(val2)
				iw.components.push_back(Component(Component::RESOURCE,Res::GOLD,val2,0));

			iw.text.addTxt(MetaString::ADVOB_TXT, 51+type);
			cb->showInfoDialog(&iw);
			break;
		}
	case Obj::SEA_CHEST:
		{
			InfoWindow iw;
			iw.soundID = soundBase::chest;
			iw.player = h->tempOwner;
			iw.text.addTxt(MetaString::ADVOB_TXT, 116 + type);

			if(val1) //there is gold
			{
				iw.components.push_back(Component(Component::RESOURCE,Res::GOLD,val1,0));
				cb->giveResource(h->tempOwner,6,val1);
			}
			if(type == 1) //art
			{
				iw.components.push_back(Component(Component::ARTIFACT, val2, 1, 0));
				iw.text.addReplacement(MetaString::ART_NAMES, val2);
				cb->giveHeroNewArtifact(h, VLC->arth->artifacts[val2],-2);
			}
			cb->showInfoDialog(&iw);
			break;
		}
	case Obj::SHIPWRECK_SURVIVOR:
		{
			InfoWindow iw;
			iw.soundID = soundBase::experience;
			iw.player = h->tempOwner;
			iw.components.push_back(Component(Component::ARTIFACT,val1,1,0));
			iw.text.addTxt(MetaString::ADVOB_TXT, 125);
			iw.text.addReplacement(MetaString::ART_NAMES, val1);
			cb->giveHeroNewArtifact(h, VLC->arth->artifacts[val1],-2);
			cb->showInfoDialog(&iw);
			break;
		}
	case Obj::TREASURE_CHEST:
		{
			if (subID) //not OH3 treasure chest
			{
				tlog2 << "Not supported WoG treasure chest!\n";
				return;
			}

			if(type) //there is an artifact
			{
				cb->giveHeroNewArtifact(h, VLC->arth->artifacts[val1],-2);
				InfoWindow iw;
				iw.soundID = soundBase::treasure;
				iw.player = h->tempOwner;
				iw.components.push_back(Component(Component::ARTIFACT,val1,1,0));
				iw.text << std::pair<ui8,ui32>(11,145);
				iw.text.addReplacement(MetaString::ART_NAMES, val1);
				cb->showInfoDialog(&iw);
				break;
			}
			else
			{
				BlockingDialog sd(false,true);
				sd.player = h->tempOwner;
				sd.text << std::pair<ui8,ui32>(11,146);
				sd.components.push_back(Component(Component::RESOURCE,Res::GOLD,val1,0));
				TExpType expVal = h->calculateXp(val2);
				sd.components.push_back(Component(Component::EXPERIENCE,0,expVal, 0));
				sd.soundID = soundBase::chest;
				boost::function<void(ui32)> fun = boost::bind(&Pickable::chosen,this,obj,_1,h->id);
				cb->showBlockingDialog(&sd,fun);
				return;
			}
		}
	}
	cb->removeObject(id);
}

void Pickable::chosen(const CGPickable* o, int which, int heroID) const
{
	const CGHeroInstance *h = cb->getHero(heroID); //TODO: use direct pointer
	switch(which)
	{
	case 1: //player pick gold
		cb->giveResource(cb->getOwner(heroID),6,o->val1);
		break;
	case 2: //player pick exp
		cb->changePrimSkill(heroID, 4, h->calculateXp(o->val2));
		break;
	default:
		throw std::runtime_error("Unhandled treasure choice");
	}
	cb->removeObject(o->id);
}

///BonusingObject
void BonusingObject::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	const auto ID = o->ID;
	const auto h = visitor;
	
	bool visited = h->hasBonusFrom(Bonus::OBJECT,ID);
	int messageID=0;
	int bonusMove = 0, sound = -1;
	InfoWindow iw;
	iw.player = h->tempOwner;
	GiveBonus gbonus;
	gbonus.id = h->id;
	gbonus.bonus.duration = Bonus::ONE_BATTLE;
	gbonus.bonus.source = Bonus::OBJECT;
	gbonus.bonus.sid = ID;

	bool second = false;
	Bonus secondBonus;

	switch(ID)
	{
	case Obj::BUOY:
		messageID = 21;
		sound = soundBase::MORALE;
		gbonus.bonus.type = Bonus::MORALE;
		gbonus.bonus.val = +1;
		gbonus.bdescr <<  std::pair<ui8,ui32>(6,94);
		break;
	case Obj::SWAN_POND:
		messageID = 29;
		sound = soundBase::LUCK;
		gbonus.bonus.type = Bonus::LUCK;
		gbonus.bonus.val = 2;
		gbonus.bdescr <<  std::pair<ui8,ui32>(6,67);
		bonusMove = -h->movement;
		break;
	case Obj::FAERIE_RING:
		messageID = 49;
		sound = soundBase::LUCK;
		gbonus.bonus.type = Bonus::LUCK;
		gbonus.bonus.val = 1;
		gbonus.bdescr <<  std::pair<ui8,ui32>(6,71);
		break;
	case Obj::FOUNTAIN_OF_FORTUNE:
		messageID = 55;
		sound = soundBase::LUCK;
		gbonus.bonus.type = Bonus::LUCK;
		gbonus.bonus.val = rand()%5 - 1;
		gbonus.bdescr <<  std::pair<ui8,ui32>(6,69);
		gbonus.bdescr.addReplacement((gbonus.bonus.val<0 ? "-" : "+") + boost::lexical_cast<std::string>(gbonus.bonus.val));
		break;
	case Obj::IDOL_OF_FORTUNE:
		messageID = 62;
		sound = soundBase::experience;

		gbonus.bonus.val = 1;
		gbonus.bdescr <<  std::pair<ui8,ui32>(6,68);
		if(cb->getDate(1) == 7) //7th day of week
		{
			gbonus.bonus.type = Bonus::MORALE;
			second = true;
			secondBonus = gbonus.bonus;
			secondBonus.type = Bonus::LUCK;
		}
		else
		{
			gbonus.bonus.type = (cb->getDate(1)%2) ? Bonus::LUCK : Bonus::MORALE;
		}
		break;
	case Obj::MERMAID:
		messageID = 83;
		sound = soundBase::LUCK;
		gbonus.bonus.type = Bonus::LUCK;
		gbonus.bonus.val = 1;
		gbonus.bdescr <<  std::pair<ui8,ui32>(6,72);
		break;
	case Obj::RALLY_FLAG:
		sound = soundBase::MORALE;
		messageID = 111;
		gbonus.bonus.type = Bonus::MORALE;
		gbonus.bonus.val = 1;
		gbonus.bdescr <<  std::pair<ui8,ui32>(6,102);

		second = true;
		secondBonus = gbonus.bonus;
		secondBonus.type = Bonus::LUCK;

		bonusMove = 400;
		break;
	case Obj::OASIS:
		messageID = 95;
		gbonus.bonus.type = Bonus::MORALE;
		gbonus.bonus.val = 1;
		gbonus.bdescr <<  std::pair<ui8,ui32>(6,95);
		bonusMove = 800;
		break;
	case Obj::TEMPLE:
		messageID = 140;
		iw.soundID = soundBase::temple;
		gbonus.bonus.type = Bonus::MORALE;
		if(cb->getDate(1)==7) //sunday
		{
			gbonus.bonus.val = 2;
			gbonus.bdescr <<  std::pair<ui8,ui32>(6,97);
		}
		else
		{
			gbonus.bonus.val = 1;
			gbonus.bdescr <<  std::pair<ui8,ui32>(6,96);
		}
		break;
	case Obj::WATERING_HOLE:
		sound = soundBase::MORALE;
		messageID = 166;
		gbonus.bonus.type = Bonus::MORALE;
		gbonus.bonus.val = 1;
		gbonus.bdescr <<  std::pair<ui8,ui32>(6,100);
		bonusMove = 400;
		break;
	case Obj::FOUNTAIN_OF_YOUTH:
		sound = soundBase::MORALE;
		messageID = 57;
		gbonus.bonus.type = Bonus::MORALE;
		gbonus.bonus.val = 1;
		gbonus.bdescr <<  std::pair<ui8,ui32>(6,103);
		bonusMove = 400;
		break;
	case Obj::STABLES:
		sound = soundBase::STORE;
		bool someUpgradeDone = false;

		for (TSlots::const_iterator i = h->Slots().begin(); i != h->Slots().end(); ++i)
		{
			if(i->second->type->idNumber == 10)
			{
				cb->changeStackType(StackLocation(h, i->first), VLC->creh->creatures[11]);
				someUpgradeDone = true;
			}
		}
		if (someUpgradeDone)
		{
			messageID = 138;
			iw.components.push_back(Component(Component::CREATURE,11,0,1));
		}
		else
			messageID = 137;

		gbonus.bonus.type = Bonus::LAND_MOVEMENT;
		gbonus.bonus.val = 600;
		bonusMove = 600;
		gbonus.bonus.duration = Bonus::ONE_WEEK;
		gbonus.bdescr <<  std::pair<ui8,ui32>(6, 100);
		break;
	}
	assert(messageID);
	if(visited)
	{
		if(ID==Obj::RALLY_FLAG || ID==Obj::OASIS || ID==Obj::MERMAID || ID==Obj::STABLES)
			messageID--;
		else
			messageID++;
	}
	else
	{
		//TODO: fix if second bonus val != main bonus val
		if(gbonus.bonus.type == Bonus::MORALE   ||   secondBonus.type == Bonus::MORALE)
			iw.components.push_back(Component(Component::MORALE,0,gbonus.bonus.val,0));
		if(gbonus.bonus.type == Bonus::LUCK   ||   secondBonus.type == Bonus::LUCK)
			iw.components.push_back(Component(Component::LUCK,0,gbonus.bonus.val,0));
		cb->giveHeroBonus(&gbonus);
		if(second)
		{
			gbonus.bonus = secondBonus;
			cb->giveHeroBonus(&gbonus);
		}
		if(bonusMove) //swan pond - take all move points, stables - give move point this day
		{
			SetMovePoints smp;
			smp.hid = h->id;
			smp.val = h->movement + bonusMove;
			cb->setMovePoints(&smp);
		}
	}
	iw.soundID = sound;
	iw.text << std::pair<ui8,ui32>(11,messageID);
	cb->showInfoDialog(&iw);
}

void MagicSpring::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	auto h = visitor;
	
	auto obj = dynamic_cast<const CGMagicSpring *>(o);
	
	int messageID;
	InfoWindow iw;
	iw.player = h->tempOwner;
	iw.soundID = soundBase::GENIE;
	if (!obj->visited)
	{
		if (h->mana > h->manaLimit())
			messageID = 76;
		else
		{
			messageID = 74;
			cb->setManaPoints (h->id, 2 * h->manaLimit());
			cb->setObjProperty (o->id, ObjProperty::VISITED, true);
		}
	}
	else
		messageID = 75;
	iw.text << std::pair<ui8,ui32>(11,messageID);
	cb->showInfoDialog(&iw);
}

void MagicWell::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	auto h = visitor;
	
	int message;
	InfoWindow iw;
	iw.soundID = soundBase::faerie;
	iw.player = h->tempOwner;
	if(h->hasBonusFrom(Bonus::OBJECT,o->ID)) //has already visited Well today
	{
		message = 78;
	}
	else if(h->mana < h->manaLimit())
	{
		giveDummyBonus(o,h->id);
		cb->setManaPoints(h->id,h->manaLimit());
		message = 77;
	}
	else
	{
		message = 79;
	}
	iw.text << std::pair<ui8,ui32>(11,message); //"A second drink at the well in one day will not help you."
	cb->showInfoDialog(&iw);
}

void PandoraBox::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	auto h = visitor;
	auto obj = dynamic_cast<const CGPandoraBox *>(o);
	
	BlockingDialog bd (true, false);
	bd.player = h->getOwner();
	bd.soundID = soundBase::QUEST;
	bd.text.addTxt (MetaString::ADVOB_TXT, 14);
	cb->showBlockingDialog (&bd, boost::bind (&PandoraBox::open, this, obj, h, _1));
}

void PandoraBox::open(const CGPandoraBox* o, const CGHeroInstance* h, ui32 accept) const
{
	if (accept)
	{
		if (o->stacksCount() > 0) //if pandora's box is protected by army
		{
			InfoWindow iw;
			iw.player = h->tempOwner;
			iw.text.addTxt(MetaString::ADVOB_TXT, 16);
			cb->showInfoDialog(&iw);
			cb->startBattleI(h, o, boost::bind(&PandoraBox::endBattle, this, o, h, _1)); //grants things after battle
		}
		else if (o->message.size() == 0 && o->resources.size() == 0
				&& o->primskills.size() == 0 && o->abilities.size() == 0
				&& o->abilityLevels.size() == 0 &&  o->artifacts.size() == 0
				&& o->spells.size() == 0 && o->creatures.Slots().size() > 0
				&& o->gainedExp == 0 && o->manaDiff == 0 && o->moraleDiff == 0 && o->luckDiff == 0) //if it gives nothing without battle
		{
			InfoWindow iw;
			iw.player = h->tempOwner;
			iw.text.addTxt(MetaString::ADVOB_TXT, 15);
			cb->showInfoDialog(&iw);

		}
		else //if it gives something without battle
		{
			giveContents (o, h, false);
		}
	}	
}

void PandoraBox::endBattle(const CGPandoraBox* o, const CGHeroInstance* h, BattleResult* result) const
{
	if(result->winner)
		return;

	giveContents(o,h,true);	
}

void PandoraBox::giveContents(const CGPandoraBox* o, const CGHeroInstance* h, bool afterBattle) const
{

	InfoWindow iw;
	iw.player = h->getOwner();
	std::string msg = o->message; //in case box is removed in the meantime

	bool changesPrimSkill = false;
	for (int i = 0; i < o->primskills.size(); i++)
	{
		if(o->primskills[i])
		{
			changesPrimSkill = true;
			break;
		}
	}

	if(o->gainedExp || changesPrimSkill || o->abilities.size())
	{
		TExpType expVal = h->calculateXp(o->gainedExp);
		//getText(iw,afterBattle,175,h); //wtf?
		iw.text.addTxt(MetaString::ADVOB_TXT, 175); //%s learns something
		iw.text.addReplacement(h->name);

		if(expVal)
			iw.components.push_back(Component(Component::EXPERIENCE,0,expVal,0));
		for(int i=0; i<o->primskills.size(); i++)
			if(o->primskills[i])
				iw.components.push_back(Component(Component::PRIM_SKILL,i,o->primskills[i],0));

		for(int i=0; i<o->abilities.size(); i++)
			iw.components.push_back(Component(Component::SEC_SKILL,o->abilities[i],o->abilityLevels[i],0));

		cb->showInfoDialog(&iw);

		//give exp
		if(expVal)
			cb->changePrimSkill(h->id,4,expVal,false);
		//give prim skills
		for(int i=0; i<o->primskills.size(); i++)
			if(o->primskills[i])
				cb->changePrimSkill(h->id,i,o->primskills[i],false);

		//give sec skills
		for(int i=0; i<o->abilities.size(); i++)
		{
			int curLev = h->getSecSkillLevel(static_cast<CGHeroInstance::SecondarySkill>(o->abilities[i]));

			if( (curLev  &&  curLev < o->abilityLevels[i])	|| (h->canLearnSkill() ))
			{
				cb->changeSecSkill(h->id,o->abilities[i],o->abilityLevels[i],true);
			}
		}

	}

	if(o->spells.size())
	{
		std::set<ui32> spellsToGive;
		iw.components.clear();
		if (o->spells.size() > 1)
		{
			iw.text.addTxt(MetaString::ADVOB_TXT, 188); //%s learns spells
		}
		else
		{
			iw.text.addTxt(MetaString::ADVOB_TXT, 184); //%s learns a spell
		}
		iw.text.addReplacement(h->name);
		std::vector<ConstTransitivePtr<CSpell> > * sp = &VLC->spellh->spells;
		for(std::vector<si32>::const_iterator i=o->spells.begin(); i != o->spells.end(); i++)
		{
			if ((*sp)[*i]->level <= h->getSecSkillLevel(CGHeroInstance::WISDOM) + 2) //enough wisdom
			{
				iw.components.push_back(Component(Component::SPELL,*i,0,0));
				spellsToGive.insert(*i);
			}
		}
		if(spellsToGive.size())
		{
			cb->changeSpells(h->id,true,spellsToGive);
			cb->showInfoDialog(&iw);
		}
	}

	if(o->manaDiff)
	{
		o->getText(iw,afterBattle,o->manaDiff,176,177,h);
		iw.components.push_back(Component(Component::PRIM_SKILL,5,o->manaDiff,0));
		cb->showInfoDialog(&iw);
		cb->setManaPoints(h->id, h->mana + o->manaDiff);
	}

	if(o->moraleDiff)
	{
		o->getText(iw,afterBattle,o->moraleDiff,178,179,h);
		iw.components.push_back(Component(Component::MORALE,0,o->moraleDiff,0));
		cb->showInfoDialog(&iw);
		GiveBonus gb;
		gb.bonus = Bonus(Bonus::ONE_BATTLE,Bonus::MORALE,Bonus::OBJECT,o->moraleDiff,o->id,"");
		gb.id = h->id;
		cb->giveHeroBonus(&gb);
	}

	if(o->luckDiff)
	{
		o->getText(iw,afterBattle,o->luckDiff,180,181,h);
		iw.components.push_back(Component(Component::LUCK,0,o->luckDiff,0));
		cb->showInfoDialog(&iw);
		GiveBonus gb;
		gb.bonus = Bonus(Bonus::ONE_BATTLE,Bonus::LUCK,Bonus::OBJECT,o->luckDiff,o->id,"");
		gb.id = h->id;
		cb->giveHeroBonus(&gb);
	}

	iw.components.clear();
	iw.text.clear();
	for(int i=0; i<o->resources.size(); i++)
	{
		if(o->resources[i] < 0)
			iw.components.push_back(Component(Component::RESOURCE,i,o->resources[i],0));
	}
	if(iw.components.size())
	{
		o->getText(iw,afterBattle,182,h);
		cb->showInfoDialog(&iw);
	}

	iw.components.clear();
	iw.text.clear();
	for(int i=0; i<o->resources.size(); i++)
	{
		if(o->resources[i] > 0)
			iw.components.push_back(Component(Component::RESOURCE,i,o->resources[i],0));
	}
	if(iw.components.size())
	{
		o->getText(iw,afterBattle,183,h);
		cb->showInfoDialog(&iw);
	}

 	iw.components.clear();
// 	getText(iw,afterBattle,183,h);
	iw.text.addTxt(MetaString::ADVOB_TXT, 183); //% has found treasure
	iw.text.addReplacement(h->name);
	for(int i=0; i<o->artifacts.size(); i++)
	{
		iw.components.push_back(Component(Component::ARTIFACT,o->artifacts[i],0,0));
		if(iw.components.size() >= 14)
		{
			cb->showInfoDialog(&iw);
			iw.components.clear();
			iw.text.addTxt(MetaString::ADVOB_TXT, 183); //% has found treasure - once more?
			iw.text.addReplacement(h->name);
		}
	}
	if(iw.components.size())
	{
		cb->showInfoDialog(&iw);
	}

	for(int i=0; i<o->resources.size(); i++)
		if(o->resources[i])
			cb->giveResource(h->getOwner(),i,o->resources[i]);

	for(int i=0; i<o->artifacts.size(); i++)
		cb->giveHeroNewArtifact(h, VLC->arth->artifacts[o->artifacts[i]],-2);

	iw.components.clear();
	iw.text.clear();

	if (o->creatures.Slots().size())
	{ //this part is taken straight from creature bank
		MetaString loot;
		for(TSlots::const_iterator i = o->creatures.Slots().begin(); i != o->creatures.Slots().end(); i++)
		{ //build list of joined creatures
			iw.components.push_back(Component(*i->second));
			loot << "%s";
			loot.addReplacement(*i->second);
		}

		if (o->creatures.Slots().size() == 1 && o->creatures.Slots().begin()->second->count == 1)
			iw.text.addTxt(MetaString::ADVOB_TXT, 185);
		else
			iw.text.addTxt(MetaString::ADVOB_TXT, 186);

		iw.text.addReplacement(loot.buildList());
		iw.text.addReplacement(h->name);

		cb->showInfoDialog(&iw);
		cb->giveCreatures(o, h, o->creatures, true);
	}
	if(!afterBattle && msg.size())
	{
		iw.text << msg;
		cb->showInfoDialog(&iw);
	}
	if (!o->creatures.Slots().size())
		cb->removeObject(o->id); //only when we don't need to display garrison window	
}

///Event
void Event::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	auto h = visitor;
	auto obj = dynamic_cast<const CGEvent *>(o);
	
	if(!(obj->availableFor & (1 << h->tempOwner)))
		return;
	if(cb->getPlayerSettings(h->tempOwner)->playerID)
	{
		if(obj->humanActivate)
			activated(obj,h);
	}
	else if(obj->computerActivate)
		activated(obj,h);	
}

void Event::activated(const CGEvent* o, const CGHeroInstance* h) const
{
	if(o->stacksCount() > 0)
	{
		InfoWindow iw;
		iw.player = h->tempOwner;
		if(o->message.size())
			iw.text << o->message;
		else
			iw.text.addTxt(MetaString::ADVOB_TXT, 16);
		cb->showInfoDialog(&iw);
		cb->startBattleI(h, o, boost::bind(&Event::endBattle,this,o,h,_1));
	}
	else
	{
		giveContents(o,h,false);
	}	
}

///Observatory
void Observatory::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	auto h=visitor;
		
	InfoWindow iw;
	iw.player = h->tempOwner;
	switch (o->ID)
	{
	case Obj::REDWOOD_OBSERVATORY:
	case Obj::PILLAR_OF_FIRE:
	{
		iw.soundID = soundBase::LIGHTHOUSE;
		iw.text.addTxt(MetaString::ADVOB_TXT,98 + (o->ID==Obj::PILLAR_OF_FIRE));

		FoWChange fw;
		fw.player = h->tempOwner;
		fw.mode = 1;
		cb->getTilesInRange (fw.tiles, o->pos, 20, h->tempOwner, 1);
		cb->sendAndApply (&fw);
		break;
	}
	case Obj::COVER_OF_DARKNESS:
	{
		iw.text.addTxt (MetaString::ADVOB_TXT, 31);
		o->hideTiles(h->tempOwner, 20);
		break;
	}
	}
	cb->showInfoDialog(&iw);	
}

///Shrine
void Shrine::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	auto obj = dynamic_cast<const CGShrine *>(o); 
	auto spell = obj->spell;
	auto h=visitor;
	
	if(spell == 255)
	{
		tlog1 << "Not initialized shrine visited!\n";
		return;
	}

	if(!o->wasVisited(h->tempOwner))
		cb->setObjProperty(o->id,10,h->tempOwner);

	InfoWindow iw;
	iw.soundID = soundBase::temple;
	iw.player = h->getOwner();
	iw.text.addTxt(MetaString::ADVOB_TXT,127 + o->ID - 88);
	iw.text.addTxt(MetaString::SPELL_NAME,spell);
	iw.text << ".";

	if(!h->getArt(ArtifactPosition::SPELLBOOK))
	{
		iw.text.addTxt(MetaString::ADVOB_TXT,131);
	}
	else if(o->ID == Obj::SHRINE_OF_MAGIC_THOUGHT  && !h->getSecSkillLevel(CGHeroInstance::WISDOM)) //it's third level spell and hero doesn't have wisdom
	{
		iw.text.addTxt(MetaString::ADVOB_TXT,130);
	}
	else if(vstd::contains(h->spells,spell))//hero already knows the spell
	{
		iw.text.addTxt(MetaString::ADVOB_TXT,174);
	}
	else //give spell
	{
		std::set<ui32> spells;
		spells.insert(spell);
		cb->changeSpells(h->id, true, spells);

		iw.components.push_back(Component(Component::SPELL,spell,0,0));
	}

	cb->showInfoDialog(&iw);
}

void SignBottle::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	auto obj = dynamic_cast<const CGSignBottle *>(o); 
	auto message = obj->message;
	auto h=visitor;
	
	InfoWindow iw;
	iw.soundID = soundBase::STORE;
	iw.player = h->getOwner();
	iw.text << message;
	cb->showInfoDialog(&iw);

	if(o->ID == Obj::OCEAN_BOTTLE)
		cb->removeObject(o->id);
}

void Scholar::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	auto obj = dynamic_cast<const CGScholar *>(o);
	auto h=visitor;
	
	int type = obj->bonusType;
	int bid = obj->bonusID;
	//check if the bonus if applicable, if not - give primary skill (always possible)
	int ssl = h->getSecSkillLevel(static_cast<CGHeroInstance::SecondarySkill>(bid)); //current sec skill level, used if bonusType == 1
	if((type == 1
			&& ((ssl == 3)  ||  (!ssl  &&  !h->canLearnSkill()))) ////hero already has expert level in the skill or (don't know skill and doesn't have free slot)
		|| (type == 2  &&  (!h->getArt(17) || vstd::contains(h->spells, (ui32) bid)
		|| (VLC->spellh->spells[bid]->level > h->getSecSkillLevel(CGHeroInstance::WISDOM) + 2)
		))) //hero doesn't have a spellbook or already knows the spell or doesn't have Wisdom
	{
		type = 0;
		bid = ran() % GameConstants::PRIMARY_SKILLS;
	}


	InfoWindow iw;
	iw.soundID = soundBase::gazebo;
	iw.player = h->getOwner();
	iw.text.addTxt(MetaString::ADVOB_TXT,115);

	switch (type)
	{
	case 0:
		cb->changePrimSkill(h->id,bid,+1);
		iw.components.push_back(Component(Component::PRIM_SKILL,bid,+1,0));
		break;
	case 1:
		cb->changeSecSkill(h->id,bid,+1);
		iw.components.push_back(Component(Component::SEC_SKILL,bid,ssl+1,0));
		break;
	case 2:
		{
			std::set<ui32> hlp;
			hlp.insert(bid);
			cb->changeSpells(h->id,true,hlp);
			iw.components.push_back(Component(Component::SPELL,bid,0,0));
		}
		break;
	default:
		tlog1 << "Error: wrong bonustype (" << (int)type << ") for Scholar!\n";
		return;
	}

	cb->showInfoDialog(&iw);
	cb->removeObject(o->id);
}


void Garrison::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	auto obj = dynamic_cast<const CGGarrison *>(o);
	auto h=visitor;
	
	int ally = cb->gameState()->getPlayerRelations(h->tempOwner, obj->tempOwner);
	if (!ally && obj->stacksCount() > 0) {
		//TODO: Find a way to apply magic garrison effects in battle.
		cb->startBattleI(h, obj, boost::bind(&Garrison::fightOver, this, obj, h, _1));
		return;
	}

	//New owner.
	if (!ally)
		cb->setOwner(obj->id, h->tempOwner);

	cb->showGarrisonDialog(obj->id, h->id, obj->removableUnits, 0);
}

void Garrison::fightOver(const CGGarrison* o, const CGHeroInstance* h, BattleResult* result)
{
	if (result->winner == 0)
		onHeroVisit(o,h);	
}


///OnceVisitable
void OnceVisitable::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	auto obj = dynamic_cast<const CGOnceVisitable *>(o);
	auto h=visitor;
	
	auto ID = obj->ID;
	auto id = obj->id;
	
	int sound;
	int txtid;

	switch(ID)
	{
	case Obj::CORPSE:
		txtid = 37;
		sound = soundBase::MYSTERY;
		break;
	case Obj::LEAN_TO:
		sound = soundBase::GENIE;
		txtid = 64;
		break;
	case Obj::WAGON:
		sound = soundBase::GENIE;
		txtid = 154;
		break;
	case Obj::WARRIORS_TOMB:
		{
			//ask if player wants to search the Tomb
			BlockingDialog bd(true, false);
			bd.soundID = soundBase::GRAVEYARD;
			bd.player = h->getOwner();
			bd.text.addTxt(MetaString::ADVOB_TXT,161);
			cb->showBlockingDialog(&bd,boost::bind(&OnceVisitable::searchTomb,this,obj,h,_1));
			return;
		}
		break;
	default:
		tlog1 << "Error: Unknown object (" << ID <<") treated as CGOnceVisitable!\n";
		return;
	}

	InfoWindow iw;
	iw.soundID = sound;
	iw.player = h->getOwner();

	if(obj->players.size()) //we have been already visited...
	{
		txtid++;
		if(ID == Obj::WAGON) //wagon has extra text (for finding art) we need to omit
			txtid++;
		iw.text.addTxt(MetaString::ADVOB_TXT, txtid);
	}
	else //first visit - give bonus!
	{
		switch(obj->artOrRes)
		{
		case 0: // first visit but empty
			if (ID == Obj::CORPSE)
				++txtid;
			else
				txtid+=2;
			iw.text.addTxt(MetaString::ADVOB_TXT, txtid);
			break;
		case 1: //art
			iw.components.push_back(Component(Component::ARTIFACT,obj->bonusType,0,0));
			cb->giveHeroNewArtifact(h, VLC->arth->artifacts[obj->bonusType],-2);
			iw.text.addTxt(MetaString::ADVOB_TXT, txtid);
			if (ID == Obj::CORPSE)
			{
				iw.text << "%s";
				iw.text.addReplacement(MetaString::ART_NAMES, obj->bonusType);
			}
			break;
		case 2: //res
			iw.text.addTxt(MetaString::ADVOB_TXT, txtid);
			iw.components.push_back (Component(Component::RESOURCE,obj->bonusType, obj->bonusVal, 0));
			cb->giveResource(h->getOwner(), obj->bonusType, obj->bonusVal);
			break;
		}
		if(ID == Obj::WAGON  &&  obj->artOrRes == 1)
		{
			iw.text.localStrings.back().second++;
			iw.text.addReplacement(MetaString::ART_NAMES, obj->bonusType);
		}
	}

	cb->showInfoDialog(&iw);
	cb->setObjProperty(id,10,h->getOwner());	
}

void OnceVisitable::searchTomb(const CGOnceVisitable* o, const CGHeroInstance* h, ui32 accept) const
{
	if(accept)
	{
		InfoWindow iw;
		iw.player = h->getOwner();
		iw.components.push_back(Component(Component::MORALE,0,-3,0));

		if(o->players.size()) //we've been already visited, player found nothing
		{
			iw.text.addTxt(MetaString::ADVOB_TXT,163);
		}
		else //first visit - give artifact
		{
			iw.text.addTxt(MetaString::ADVOB_TXT,162);
			iw.components.push_back(Component(Component::ARTIFACT,o->bonusType,0,0));
			iw.text.addReplacement(MetaString::ART_NAMES, o->bonusType);

			cb->giveHeroNewArtifact(h, VLC->arth->artifacts[o->bonusType],-2);
		}

		if(!h->hasBonusFrom(Bonus::OBJECT,o->ID)) //we don't have modifier from this object yet
		{
			//ruin morale
			GiveBonus gb;
			gb.id = h->id;
			gb.bonus = Bonus(Bonus::ONE_BATTLE,Bonus::MORALE,Bonus::OBJECT,-3,o->id,"");
			gb.bdescr.addTxt(MetaString::ARRAY_TXT,104); //Warrior Tomb Visited -3
			cb->giveHeroBonus(&gb);
		}
		cb->showInfoDialog(&iw);
		cb->setObjProperty(o->id,10,h->getOwner());
	}	
}

void Bank::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	
}

void Bank::fightGuards(const CBank* o, const CGHeroInstance* h, ui32 accept)
{
	
}

void Bank::endBattle(const CBank* o, const CGHeroInstance* h, const BattleResult* result)
{
	
}

void Pyramid::onHeroVisit(const CGObjectInstance* o, const CGHeroInstance* visitor)
{
	
}

void Pyramid::endBattle(const CBank* o, const CGHeroInstance* h, const BattleResult* result)
{
	
}


///IEventDispatcher factory
IEventDispatcher * IEventDispatcher::createDispatcher(IGameCallback * cb)
{
	return new CEventDispatcher(cb);
}

///CEventDispatcher
CEventDispatcher::CEventDispatcher(IGameCallback* cb)
{
	#define HANDLER_LIST \
	HANDLER(HILL_FORT,HillFort)\
	HANDLER(SANCTUARY,Sanctuary)\
	HANDLER(TAVERN,Tavern)\
	HANDLER(HERO,HeroI)\
	HANDLER(PRISON,Prison)\
	HANDLER(TOWN,Town)\
	HANDLER(CREATURE_GENERATOR1,Dwelling)\
	HANDLER(CREATURE_GENERATOR2,Dwelling)\
	HANDLER(CREATURE_GENERATOR3,Dwelling)\
	HANDLER(CREATURE_GENERATOR4,Dwelling)\
	HANDLER(REFUGEE_CAMP,Dwelling)\
	HANDLER(WAR_MACHINE_FACTORY,Dwelling)\
	HANDLER(ARENA,VisitableOPH)\
	HANDLER(MERCENARY_CAMP,VisitableOPH)\
	HANDLER(MARLETTO_TOWER,VisitableOPH)\
	HANDLER(STAR_AXIS,VisitableOPH)\
	HANDLER(GARDEN_OF_REVELATION,VisitableOPH)\
	HANDLER(LEARNING_STONE,VisitableOPH)\
	HANDLER(TREE_OF_KNOWLEDGE,VisitableOPH)\
	HANDLER(LIBRARY_OF_ENLIGHTENMENT,VisitableOPH)\
	HANDLER(SCHOOL_OF_MAGIC,VisitableOPH)\
	HANDLER(SCHOOL_OF_WAR,VisitableOPH)\
	HANDLER(MINE,Mine)\
	HANDLER(ABANDONED_MINE,Mine)\
	HANDLER(MONSTER,Creature)\
	HANDLER(RESOURCE,Resource)\
	HANDLER(MYSTICAL_GARDEN,VisitableOPW)\
	HANDLER(WINDMILL,VisitableOPW)\
	HANDLER(WATER_WHEEL,VisitableOPW)\
	HANDLER(MONOLITH1,Teleport)\
	HANDLER(MONOLITH2,Teleport)\
	HANDLER(MONOLITH3,Teleport)\
	HANDLER(SUBTERRANEAN_GATE,Teleport)\
	HANDLER(WHIRLPOOL,Teleport)\
	HANDLER(ARTIFACT,Artifact)\
	HANDLER(SPELL_SCROLL,Artifact)\
	HANDLER(CAMPFIRE,Pickable)\
	HANDLER(FLOTSAM,Pickable)\
	HANDLER(SEA_CHEST,Pickable)\
	HANDLER(SHIPWRECK_SURVIVOR,Pickable)\
	HANDLER(TREASURE_CHEST,Pickable)\
	HANDLER(FAERIE_RING,BonusingObject)\
	HANDLER(SWAN_POND,BonusingObject)\
	HANDLER(IDOL_OF_FORTUNE,BonusingObject)\
	HANDLER(FOUNTAIN_OF_FORTUNE,BonusingObject)\
	HANDLER(RALLY_FLAG,BonusingObject)\
	HANDLER(OASIS,BonusingObject)\
	HANDLER(TEMPLE,BonusingObject)\
	HANDLER(WATERING_HOLE,BonusingObject)\
	HANDLER(FOUNTAIN_OF_YOUTH,BonusingObject)\
	HANDLER(BUOY,BonusingObject)\
	HANDLER(MERMAID,BonusingObject)\
	HANDLER(STABLES,BonusingObject)\
	HANDLER(MAGIC_SPRING,MagicSpring)\
	HANDLER(MAGIC_WELL,MagicWell)\
	HANDLER(PANDORAS_BOX,PandoraBox)\
	HANDLER(EVENT,Event)\
	HANDLER(COVER_OF_DARKNESS,Observatory)\
	HANDLER(REDWOOD_OBSERVATORY,Observatory)\
	HANDLER(PILLAR_OF_FIRE,Observatory)\
	HANDLER(SHRINE_OF_MAGIC_INCANTATION,Shrine)\
	HANDLER(SHRINE_OF_MAGIC_GESTURE,Shrine)\
	HANDLER(SHRINE_OF_MAGIC_THOUGHT,Shrine)\
	HANDLER(OCEAN_BOTTLE,SignBottle)\
	HANDLER(SIGN,SignBottle)\
	HANDLER(SCHOLAR,Scholar)\
	HANDLER(GARRISON,Garrison)\
	HANDLER(GARRISON2,Garrison)\
	HANDLER(CORPSE,OnceVisitable)\
	HANDLER(LEAN_TO,OnceVisitable)\
	HANDLER(WAGON,OnceVisitable)\
	HANDLER(WARRIORS_TOMB,OnceVisitable)\
	HANDLER(CREATURE_BANK,Bank)\
	HANDLER(DERELICT_SHIP,Bank)\
	HANDLER(DRAGON_UTOPIA,Bank)\
	HANDLER(CRYPT,Bank)\
	HANDLER(SHIPWRECK,Bank)\
	HANDLER(PYRAMID,Pyramid)\

	CB local_cb;
	local_cb.cb = cb;
	local_cb.dispatcher = this;

	#define HANDLER(Id,Class) (Obj::Id, shared_ptr<IServerObject>(new Class(local_cb)))
	std::map<Obj::Obj,THandlerPtr> h = boost::assign::map_list_of HANDLER_LIST;
	h.swap(handlers);
	#undef HANDLER
	#undef HANDLER_LIST
}
	
void CEventDispatcher::onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor)
{
	const Obj::Obj ID = (Obj::Obj)o->ID; 
	
	if (handlers.find(ID)==handlers.end())
	{
		//not found 		
		o->onHeroVisit(visitor);
	}
	else if (ID==Obj::WOG_OBJECT && o->subID)
	{
		//WoG Object, not PYRAMID
		//TODO: handle WoG Object 
		
		//TODO: maybe use map of subids
	}	
	else
	{
		handlers[ID]->onHeroVisit(o,visitor);
	}	
}


}//namespace server_objects

namespace hero_mechanics{

CStackBasicDescriptor calculateNecromancy (const CGHeroInstance *hero, const BattleResult &battleResult) 
{
	if (!hero)
	{
		return CStackBasicDescriptor();	
	}
	const ui8 necromancyLevel = hero->getSecSkillLevel(CGHeroInstance::NECROMANCY);

	// Hero knows necromancy.
	if (necromancyLevel > 0)
	{
		double necromancySkill = hero->valOfBonuses(Bonus::SECONDARY_SKILL_PREMY, CGHeroInstance::NECROMANCY)/100.0;
		vstd::amin(necromancySkill, 1.0); //it's impossible to raise more creatures than all...
		const std::map<ui32,si32> &casualties = battleResult.casualties[!battleResult.winner];
		ui32 raisedUnits = 0;

		// Figure out what to raise and how many.
		const ui32 creatureTypes[] = {56, 58, 60, 64}; // IDs for Skeletons, Walking Dead, Wights and Liches respectively.
		const bool improvedNecromancy = hero->hasBonusOfType(Bonus::IMPROVED_NECROMANCY);
		const CCreature *raisedUnitType = VLC->creh->creatures[creatureTypes[improvedNecromancy ? necromancyLevel : 0]];
		const ui32 raisedUnitHP = raisedUnitType->valOfBonuses(Bonus::STACK_HEALTH);

		//calculate creatures raised from each defeated stack
		BOOST_FOREACH(auto it, casualties)
		{
			// Get lost enemy hit points convertible to units.
			CCreature * c = VLC->creh->creatures[it.first];
			if (c->isLiving())
			{
				const ui32 raisedHP = c->valOfBonuses(Bonus::STACK_HEALTH) * it.second * necromancySkill;
				raisedUnits += std::min<ui32>(raisedHP / raisedUnitHP, it.second * necromancySkill); //limit to % of HP and % of original stack count
			}
		}

		// Make room for new units.
		int slot = hero->getSlotFor(raisedUnitType->idNumber);
		if (slot == -1)
		{
			// If there's no room for unit, try it's upgraded version 2/3rds the size.
			raisedUnitType = VLC->creh->creatures[*raisedUnitType->upgrades.begin()];
			raisedUnits = (raisedUnits*2)/3;

			slot = hero->getSlotFor(raisedUnitType->idNumber);
		}
		if (raisedUnits <= 0)
			raisedUnits = 1;

		return CStackBasicDescriptor(raisedUnitType->idNumber, raisedUnits);
	}

	return CStackBasicDescriptor();
}

void prepareNecromancyDialog(const CGHeroInstance *hero, const CStackBasicDescriptor &raisedStack, InfoWindow &iw)
{
	iw.soundID = soundBase::pickup01 + ran() % 7;
	iw.player = hero->tempOwner;
	iw.components.push_back(Component(raisedStack));

	if (raisedStack.count > 1) // Practicing the dark arts of necromancy, ... (plural)
	{
		iw.text.addTxt(MetaString::GENERAL_TXT, 145);
		iw.text.addReplacement(raisedStack.count);
		iw.text.addReplacement(MetaString::CRE_PL_NAMES, raisedStack.type->idNumber);
	}
	else // Practicing the dark arts of necromancy, ... (singular)
	{
		iw.text.addTxt(MetaString::GENERAL_TXT, 146);
		iw.text.addReplacement(MetaString::CRE_SING_NAMES, raisedStack.type->idNumber);
	}	
}

//TODO: move  to GameHandler	
void showNecromancyDialog(const CGHeroInstance *hero, const CStackBasicDescriptor &raisedStack)
{
	InfoWindow iw;
	
	prepareNecromancyDialog(hero,raisedStack,iw);

	IObjectInterface::cb->showInfoDialog(&iw); 
	
}
} //namespace hero_mechanics
 